using AutoMapper;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;
using Tsi1.DataLayer;
using Tsi1.DataLayer.Entities;

namespace Tsi1.BusinessLayer.Services
{
    public class TenantService : ITenantService
    {
        private readonly Tsi1Context _context;

        private readonly IMapper _mapper;

        public TenantService(Tsi1Context context, IMapper mapper)
        {
            _context = context;
            _mapper = mapper;
        }

        public async Task<ServiceResult<List<TenantPreviewDto>>> GetAll()
        {
            var result = new ServiceResult<List<TenantPreviewDto>>();

            var tenants = await _context.Tenants.ToListAsync();

            var tenantsDto = _mapper.Map<List<TenantPreviewDto>>(tenants);

            result.Data = tenantsDto;

            return result;
        }

        public async Task<ServiceResult<int>> GetByName(string tenantName)
        {
            var result = new ServiceResult<int>();

            var tenant = await _context.Tenants
                .FirstOrDefaultAsync(x => x.Name == tenantName);

            if (tenant == null)
            {
                result.HasError = true;
                result.Message = string.Format(ErrorMessages.TenantDoesNotExist, tenantName);
                return result;
            }

            result.Data = tenant.Id;

            return result;
        }

        public async Task<ServiceResult<int>> Create(TenantCreateDto newTenant)
        {
            var result = new ServiceResult<int>();

            var existingTenant = await _context.Tenants.FirstOrDefaultAsync(x => x.Name == newTenant.Name);

            if (existingTenant != null)
            {
                result.HasError = true;
                result.Message = string.Format(ErrorMessages.DuplicateTenantName, newTenant.Name);
                return result;
            }

            var tenant = _mapper.Map<Tenant>(newTenant);
            _context.Tenants.Add(tenant);

            await _context.SaveChangesAsync();
         
            result.Data = tenant.Id;
            return result;
        }

        public async Task<ServiceResult<bool>> Modify(int tenantId, TenantCreateDto tenantDto)
        {
            var result = new ServiceResult<bool>();

            var tenant = await _context.Tenants
                .FirstOrDefaultAsync(x => x.Id == tenantId);

            if (tenant == null)
            {
                result.Message = string.Format(ErrorMessages.TenantDoesNotExist, tenantId);
                return result;
            }

            _mapper.Map(tenantDto, tenant);

            await _context.SaveChangesAsync();

            result.Data = true;

            return result;
        }

        public async Task<ServiceResult<bool>> Delete(int tenantId)
        {
            var result = new ServiceResult<bool>();

            var tenant = await _context.Tenants
                .FirstOrDefaultAsync(x => x.Id == tenantId);

            if (tenant == null)
            {
                result.Message = string.Format(ErrorMessages.TenantDoesNotExist, tenantId);
                return result;
            }

            _context.Tenants.Remove(tenant);

            await _context.SaveChangesAsync();

            result.Data = true;

            return result;
        }

        public async Task<ServiceResult<List<TenantPreviewDto>>> FacultyList()
        {
            var result = new ServiceResult<List<TenantPreviewDto>>();

            var tenants = await _context.Tenants
                .Include(x => x.Students)
                .Where(x => x.Name != TenantAdmin.Name)
                .ToListAsync();

            var tenantDtos = new List<TenantPreviewDto>();
            foreach (var tenant in tenants)
            {
                var tenantDto = _mapper.Map<TenantPreviewDto>(tenant);
                tenantDto.StudentQuantity = tenant.Students.Count();

                tenantDtos.Add(tenantDto);
            }

            result.Data = tenantDtos;
            return result;
        }

        public async Task<ServiceResult<List<TenantCourseDto>>> CourseList()
        {
            var result = new ServiceResult<List<TenantCourseDto>>();

            var tenants = await _context.Tenants
                .Include(x => x.Courses)
                    .ThenInclude(x => x.StudentCourses)
                .Where(x => x.Name != TenantAdmin.Name)
                .ToListAsync();

            var tenantDtos = new List<TenantCourseDto>();
            foreach (var tenant in tenants)
            {
                var tenantDto = _mapper.Map<TenantCourseDto>(tenant);

                foreach (var course in tenant.Courses)
                {
                    var courseDto = _mapper.Map<CoursePreviewDto>(course);
                    courseDto.StudentQuantity = course.StudentCourses.Count();

                    tenantDto.Courses.Add(courseDto);
                }

                tenantDtos.Add(tenantDto);
            }

            result.Data = tenantDtos;
            return result;
        }

        public async Task<ServiceResult<int>> GetById(int tenantId)
        {
            var result = new ServiceResult<int>();

            var tenant = await _context.Tenants
                .FirstOrDefaultAsync(x => x.Id == tenantId);

            if (tenant == null)
            {
                result.HasError = true;
                result.Message = string.Format(ErrorMessages.TenantDoesNotExist, tenantId);
                return result;
            }

            result.Data = tenant.Id;

            return result;
        }
    }
}
